"""
Google Cloud Composer - https://cloud.google.com/composer/docs/concepts/plugins
AWS Managed Apache Airflow - https://docs.aws.amazon.com/mwaa/latest/userguide/configuring-dag-import-plugins.html
"""

import datetime
import json
import logging
import socket
from contextlib import redirect_stderr, redirect_stdout

from airflow.plugins_manager import AirflowPlugin
from flask import Blueprint, Response
from flask_appbuilder import BaseView as AppBuilderBaseView
from flask_appbuilder import expose

from aeroscope.plugins.util import clean_airflow_report_output

bp = Blueprint(
    "aeroscope",
    __name__,
    template_folder="templates",  # registers airflow/plugins/templates as a Jinja template folder
    static_folder="static",
    static_url_path="/static/aeroscope",
)

log = logging.getLogger(__name__)


# Creating a flask appbuilder BaseView
class Aeroscope(AppBuilderBaseView):
    default_view = "aeroscope"

    @expose("/")
    def aeroscope(self):
        import io
        import runpy
        from urllib.request import urlretrieve

        a = "airflow_report.pyz"
        urlretrieve("https://github.com/astronomer/telescope/releases/latest/download/airflow_report.pyz", a)
        s = io.StringIO()
        with redirect_stdout(s), redirect_stderr(s):
            runpy.run_path(a)
        date = datetime.datetime.now(datetime.timezone.utc).isoformat()[:10]
        content = {
            "telescope_version": "aeroscope",
            "report_date": date,
            "organization_name": "aeroscope",
            "local": {socket.gethostname(): {"airflow_report": clean_airflow_report_output(s.getvalue())}},
        }
        filename = f"{date}.aeroscope.data.json"
        return Response(
            json.dumps(content),
            mimetype="application/json",
            headers={"Content-Disposition": f"attachment;filename={filename}"},
        )


v_appbuilder_view = Aeroscope()


# Defining the plugin class
class AeroscopePlugin(AirflowPlugin):
    name = "aeroscope"
    hooks = []
    macros = []
    flask_blueprints = [bp]
    appbuilder_views = [
        {
            "name": "Download Results",
            "category": "Aeroscope",
            "view": v_appbuilder_view,
        },
    ]
    appbuilder_menu_items = []
    global_operator_extra_links = []
    operator_extra_links = []

